package lcsb.mapviewer.model.job;

import java.io.Serializable;
import java.util.Calendar;
import java.util.Collections;
import java.util.Map;

import javax.persistence.Column;
import javax.persistence.Convert;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;

import com.fasterxml.jackson.annotation.JsonIgnore;

import lcsb.mapviewer.modelutils.serializer.JpaMapConverterJson;

@Entity
public class MinervaJob implements Serializable {
  /**
   * 
   */
  private static final long serialVersionUID = 1L;

  @Id
  @GeneratedValue(strategy = GenerationType.IDENTITY)
  @JsonIgnore
  private int id;

  @Column(nullable = false)
  @Enumerated(EnumType.STRING)
  private MinervaJobType jobType;

  @Column(nullable = false)
  @Enumerated(EnumType.STRING)
  private MinervaJobStatus jobStatus = MinervaJobStatus.PENDING;

  @Convert(converter = JpaMapConverterJson.class)
  private Map<String, Object> jobParameters;

  private Calendar jobStarted;

  private Calendar jobFinished;

  private String logs;

  private int priority = MinervaJobPriority.LOWEST.getValue();

  protected MinervaJob() {

  }

  public MinervaJob(final MinervaJobType jobType, final MinervaJobPriority priority, final MinervaJobParameters jobParameters) {
    this.jobType = jobType;
    this.priority = priority.getValue();
    this.jobParameters = jobParameters.getJobParameters();
  }

  public MinervaJobType getJobType() {
    return jobType;
  }

  public int getId() {
    return id;
  }

  public Map<String, Object> getJobParameters() {
    return Collections.unmodifiableMap(jobParameters);
  }

  public MinervaJobStatus getJobStatus() {
    return jobStatus;
  }

  public void setJobStatus(final MinervaJobStatus jobStatus) {
    if (this.jobStatus != jobStatus) {
      if (jobStatus == MinervaJobStatus.RUNNING) {
        this.jobStarted = Calendar.getInstance();
      }
      if (jobStatus == MinervaJobStatus.FAILED
          || jobStatus == MinervaJobStatus.DONE
          || jobStatus == MinervaJobStatus.INTERRUPTED) {
        this.jobFinished = Calendar.getInstance();
      }
    }
    this.jobStatus = jobStatus;
  }

  public Calendar getJobStarted() {
    return jobStarted;
  }

  public Calendar getJobFinished() {
    return jobFinished;
  }

  public String getLogs() {
    return logs;
  }

  public void setLogs(final String logs) {
    this.logs = logs;
  }

  public int getPriority() {
    return priority;
  }

}
