package lcsb.mapviewer.services.impl;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import org.hibernate.Hibernate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import lcsb.mapviewer.model.map.MiriamType;
import lcsb.mapviewer.model.map.model.ModelData;
import lcsb.mapviewer.model.map.species.Element;
import lcsb.mapviewer.persist.dao.map.species.ElementDao;
import lcsb.mapviewer.persist.dao.map.species.ElementProperty;
import lcsb.mapviewer.services.ObjectNotFoundException;
import lcsb.mapviewer.services.interfaces.IElementService;
import lcsb.mapviewer.services.interfaces.IModelService;

@Transactional
@Service
public class ElementService implements IElementService {

  @Autowired
  private IModelService modelService;

  @Autowired
  private ElementDao elementDao;

  @Override
  public Element getElementById(final String projectId, final int mapId, final int elementId) throws ObjectNotFoundException {
    ModelData map = modelService.getModelByMapId(projectId, mapId);
    Map<ElementProperty, List<?>> properties = new HashMap<>();
    properties.put(ElementProperty.ID, Arrays.asList(elementId));
    properties.put(ElementProperty.MAP, Arrays.asList(map));
    Page<Element> elements = elementDao.getByFilter(Pageable.unpaged(), properties);
    if (elements.getNumberOfElements() > 0) {
      return elements.getContent().get(0);
    }
    throw new ObjectNotFoundException("Element does not exist");
  }

  @Override
  public Element getElementById(final String projectId, final int elementId) throws ObjectNotFoundException {
    List<ModelData> maps = modelService.getModelsByProjectId(projectId);
    if (maps.size() == 0) {
      throw new ObjectNotFoundException("Element does not exist");
    }
    Map<ElementProperty, List<?>> properties = new HashMap<>();
    properties.put(ElementProperty.ID, Arrays.asList(elementId));
    properties.put(ElementProperty.MAP, maps);
    Page<Element> elements = elementDao.getByFilter(Pageable.unpaged(), properties);
    if (elements.getNumberOfElements() > 0) {
      return elements.getContent().get(0);
    }
    throw new ObjectNotFoundException("Element does not exist");
  }

  @Override
  public Element getById(final int id) {
    return elementDao.getById(id);
  }

  @Override
  public List<Element> getElementsByFilter(final Map<ElementProperty, List<? extends Object>> properties) {
    return elementDao.getByFilter(Pageable.unpaged(), properties).getContent();
  }

  @Override
  public List<Element> getElementsByFilter(final Map<ElementProperty, List<? extends Object>> properties, final boolean initializeLazy) {
    List<Element> result = getElementsByFilter(properties);
    if (initializeLazy) {
      for (final Element element : result) {
        if (element.getSubmodel() != null) {
          Hibernate.initialize(element.getSubmodel().getSubmodel());
        }
        Hibernate.initialize(element.getGlyph());
        Hibernate.initialize(element.getModel());
      }
    }
    return result;
  }

  @Override
  public Map<MiriamType, Integer> getAnnotationStatistics(final String projectId, final String mapId) throws ObjectNotFoundException {
    Map<MiriamType, Integer> elementAnnotations = new TreeMap<>();
    Map<MiriamType, Integer> data = elementDao.getAnnotationStatistics(modelService.getModelsByMapId(projectId, mapId));
    for (final MiriamType mt : MiriamType.values()) {
      if (data.get(mt) != null) {
        elementAnnotations.put(mt, data.get(mt));
      } else {
        elementAnnotations.put(mt, 0);
      }
    }
    return elementAnnotations;
  }

}
