package lcsb.mapviewer.services.impl;

import com.icegreen.greenmail.util.GreenMail;
import lcsb.mapviewer.model.security.Privilege;
import lcsb.mapviewer.model.security.PrivilegeType;
import lcsb.mapviewer.model.user.ConfigurationElementType;
import lcsb.mapviewer.model.user.User;
import lcsb.mapviewer.services.ServiceTestFunctionsNoTransaction;
import lcsb.mapviewer.services.interfaces.IUserService;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

public class UserServiceNoTransactionTest extends ServiceTestFunctionsNoTransaction {

  private static final String TEST_LOGIN_1 = "test_user_1";

  private static final String TEST_LOGIN_2 = "test_user_2";

  @Autowired
  private IUserService userService;

  private GreenMail mailServer;

  @Before
  public void setUp() throws Exception {
    mailServer = new GreenMail();
    mailServer.start();
    configurationService.setConfigurationValue(ConfigurationElementType.EMAIL_SMTP_SERVER, "127.0.0.1");
    configurationService.setConfigurationValue(ConfigurationElementType.EMAIL_SMTP_PORT, mailServer.getSmtp().getPort() + "");
    mailServer.setUser(configurationService.getConfigurationValue(ConfigurationElementType.EMAIL_LOGIN),
        configurationService.getConfigurationValue(ConfigurationElementType.EMAIL_PASSWORD));
  }

  @After
  public void tearDown() throws Exception {
    mailServer.stop();
    configurationService.setConfigurationValue(ConfigurationElementType.EMAIL_SMTP_SERVER, "");
    User user1 = userService.getUserByLogin(TEST_LOGIN_1);
    if (user1 != null) {
      userService.deleteUser(user1);
    }
    User user2 = userService.getUserByLogin(TEST_LOGIN_1);
    if (user2 != null) {
      userService.deleteUser(user2);
    }
  }


  @Test
  public void revokePrivilegeGloballyWorks() {
    User user1 = createTestUser(TEST_LOGIN_1);
    userService.addUser(user1);

    User user2 = createTestUser(TEST_LOGIN_2);
    userService.addUser(user2);

    Privilege privilege = new Privilege(PrivilegeType.WRITE_PROJECT, "foo");

    user1 = userService.getUserByLogin(user1.getLogin(), true);
    user2 = userService.getUserByLogin(user2.getLogin(), true);

    assertFalse(user1.getPrivileges().contains(privilege));
    assertFalse(user2.getPrivileges().contains(privilege));

    userService.grantUserPrivilege(user1, PrivilegeType.WRITE_PROJECT, "foo");
    userService.grantUserPrivilege(user2, PrivilegeType.WRITE_PROJECT, "foo");

    user1 = userService.getUserByLogin(user1.getLogin(), true);
    user2 = userService.getUserByLogin(user2.getLogin(), true);
    assertTrue(user1.getPrivileges().contains(privilege));
    assertTrue(user2.getPrivileges().contains(privilege));

    userService.revokeObjectDomainPrivilegesForAllUsers(PrivilegeType.WRITE_PROJECT, "foo");

    user1 = userService.getUserByLogin(user1.getLogin(), true);
    user2 = userService.getUserByLogin(user2.getLogin(), true);

    assertFalse(user1.getPrivileges().contains(privilege));
    assertFalse(user2.getPrivileges().contains(privilege));
  }
}
