package lcsb.mapviewer.web.api;

import java.awt.Color;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.util.List;
import java.util.Objects;

import lcsb.mapviewer.api.LayoutChangeNotSupportedException;
import lcsb.mapviewer.api.QueryException;
import lcsb.mapviewer.common.comparator.BooleanComparator;
import lcsb.mapviewer.common.comparator.ColorComparator;
import lcsb.mapviewer.common.comparator.DoubleComparator;
import lcsb.mapviewer.common.comparator.EnumComparator;
import lcsb.mapviewer.common.comparator.IntegerComparator;
import lcsb.mapviewer.common.comparator.LineComparator;
import lcsb.mapviewer.common.comparator.ListComparator;
import lcsb.mapviewer.common.comparator.PointComparator;
import lcsb.mapviewer.common.comparator.StringComparator;
import lcsb.mapviewer.model.graphics.ArrowTypeData;
import lcsb.mapviewer.model.graphics.ArrowTypeDataComparator;

public class AbstractDTO {
  private static DoubleComparator doubleComparator = new DoubleComparator();
  private static IntegerComparator integerComparator = new IntegerComparator();
  private static StringComparator stringComparator = new StringComparator();
  private static PointComparator pointComparator = new PointComparator();
  private static ColorComparator colorComparator = new ColorComparator();
  private static BooleanComparator booleanComparator = new BooleanComparator();
  private static ArrowTypeDataComparator atdComparator = new ArrowTypeDataComparator();
  private static ListComparator<Line2D> linesComparator = new ListComparator<>(new LineComparator());
  @SuppressWarnings("rawtypes")
  private static EnumComparator enumComparator = new EnumComparator<>();

  protected void checkEquality(final String fieldName, final Double newValue, final Double origianlValue) throws LayoutChangeNotSupportedException {
    if (doubleComparator.compare(newValue, origianlValue) != 0) {
      throw new LayoutChangeNotSupportedException(fieldName, newValue, origianlValue);
    }
  }

  protected void checkEquality(final String fieldName, final Integer newValue, final Integer origianlValue) throws LayoutChangeNotSupportedException {
    if (integerComparator.compare(newValue, origianlValue) != 0) {
      throw new LayoutChangeNotSupportedException(fieldName, newValue, origianlValue);
    }
  }

  protected void checkEquality(final String fieldName, final Color newValue, final Color origianlValue) throws LayoutChangeNotSupportedException {
    if (colorComparator.compare(newValue, origianlValue) != 0) {
      throw new LayoutChangeNotSupportedException(fieldName, newValue, origianlValue);
    }
  }

  protected void checkEquality(final String fieldName, final String newValue, final String origianlValue) throws LayoutChangeNotSupportedException {
    if (stringComparator.compare(newValue, origianlValue) != 0) {
      throw new LayoutChangeNotSupportedException(fieldName, newValue, origianlValue);
    }
  }

  protected void checkEquality(final String fieldName, final Point2D newValue, final Point2D origianlValue) throws LayoutChangeNotSupportedException {
    if (pointComparator.compare(newValue, origianlValue) != 0) {
      throw new LayoutChangeNotSupportedException(fieldName, newValue, origianlValue);
    }
  }

  protected void checkEquality(final String fieldName, final Boolean newValue, final Boolean origianlValue) throws LayoutChangeNotSupportedException {
    if (booleanComparator.compare(newValue, origianlValue) != 0) {
      throw new LayoutChangeNotSupportedException(fieldName, newValue, origianlValue);
    }
  }

  protected void checkEquality(final String fieldName, final Class<?> newValue, final Class<?> origianlValue)
      throws lcsb.mapviewer.api.QueryException {
    if (!Objects.equals(newValue, origianlValue)) {
      throw new QueryException("Cannot change: " + fieldName);
    }
  }

  @SuppressWarnings("unchecked")
  protected void checkEquality(final String fieldName, final Enum<?> newValue, final Enum<?> origianlValue) throws LayoutChangeNotSupportedException {
    if (enumComparator.compare(newValue, origianlValue) != 0) {
      throw new LayoutChangeNotSupportedException(fieldName, newValue, origianlValue);
    }
  }

  protected void checkEquality(final String fieldName, final ArrowTypeData newValue, final ArrowTypeData oldValue)
      throws LayoutChangeNotSupportedException {
    if (atdComparator.compare(newValue, oldValue) != 0) {
      throw new LayoutChangeNotSupportedException(fieldName, newValue, oldValue);
    }
  }

  protected void checkEquality(final String fieldName, final List<Line2D> newValue, final List<Line2D> oldValue)
      throws LayoutChangeNotSupportedException {
    if (linesComparator.compare(newValue, oldValue) != 0) {
      throw new LayoutChangeNotSupportedException(fieldName, newValue, oldValue);
    }
  }

}
