package lcsb.mapviewer.web.api.project;

import lcsb.mapviewer.model.Project;
import lcsb.mapviewer.web.ControllerIntegrationTest;
import lcsb.mapviewer.web.api.NewApiDocs;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.mock.web.MockHttpSession;
import org.springframework.restdocs.payload.FieldDescriptor;
import org.springframework.restdocs.payload.JsonFieldType;
import org.springframework.restdocs.snippet.Snippet;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.test.web.servlet.RequestBuilder;

import java.util.ArrayList;
import java.util.List;

import static org.springframework.restdocs.mockmvc.MockMvcRestDocumentation.document;
import static org.springframework.restdocs.mockmvc.RestDocumentationRequestBuilders.get;
import static org.springframework.restdocs.payload.PayloadDocumentation.fieldWithPath;
import static org.springframework.restdocs.payload.PayloadDocumentation.responseFields;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@RunWith(SpringJUnit4ClassRunner.class)
@ActiveProfiles("webCtdTestProfile")
public class NewProjectControllerTest extends ControllerIntegrationTest {

  private static final String TEST_PROJECT = "TEST_PROJECT";

  private Project project;

  @Before
  public void setup() {
    project = createAndPersistProject(TEST_PROJECT);
  }

  @After
  public void tearDown() throws Exception {
    if (project != null) {
      removeProject(project);
    }
  }

  @Test
  public void testGetProject() throws Exception {
    final MockHttpSession session = createSession(BUILT_IN_TEST_ADMIN_LOGIN, BUILT_IN_TEST_ADMIN_PASSWORD);

    final RequestBuilder request = get("/minerva/new_api/projects/{projectId}/", TEST_PROJECT)
        .session(session);

    mockMvc.perform(request)
        .andDo(document("new_api/projects/get_by_project_id",
            getProjectPathParameters(),
            responseFields(NewApiDocs.getProjectResponse(""))))
        .andExpect(status().is2xxSuccessful())
        .andReturn().getResponse().getContentAsString();
  }

  @Test
  public void testGetProjects() throws Exception {
    final MockHttpSession session = createSession(BUILT_IN_TEST_ADMIN_LOGIN, BUILT_IN_TEST_ADMIN_PASSWORD);

    final RequestBuilder request = get("/minerva/new_api/projects/")
        .session(session);

    final String content = mockMvc.perform(request)
        .andDo(document("new_api/projects/get_projects",
            getProjectsSearchResult()))
        .andExpect(status().is2xxSuccessful())
        .andReturn().getResponse().getContentAsString();
    logger.debug(content);
  }

  private Snippet getProjectsSearchResult() {
    final String prefix = "content[].";
    final List<FieldDescriptor> fields = new ArrayList<>();
    fields.add(
        fieldWithPath("content")
            .description("list of projects on the page")
            .type(JsonFieldType.ARRAY));
    fields.addAll(NewApiDocs.getProjectResponse(prefix));
    fields.addAll(NewApiDocs.getPageableFields());
    return responseFields(fields);
  }

}
