package lcsb.mapviewer.web.config;

import lcsb.mapviewer.common.Configuration;
import lcsb.mapviewer.model.user.User;
import lcsb.mapviewer.services.interfaces.IUserService;
import lcsb.mapviewer.web.ControllerIntegrationTest;
import org.junit.After;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import static org.junit.Assert.assertEquals;

@RunWith(SpringJUnit4ClassRunner.class)
public class UserDetailsServiceImplTest extends ControllerIntegrationTest {

  static final String TEST_LOGIN = "testLogin";

  @Autowired
  private UserDetailsServiceImpl userDetailsService;

  @Autowired
  private IUserService userService;

  @After
  public void tearDown() throws Exception {
    removeUser(userService.getUserByLogin(TEST_LOGIN));
  }

  @Test
  public void testUserHasAnonymousPermissions() throws Exception {
    createUser(TEST_LOGIN, "def");

    UserDetails details = userDetailsService.loadUserByUsername(Configuration.ANONYMOUS_LOGIN);
    User anonymous = userService.getUserByLogin(Configuration.ANONYMOUS_LOGIN, true);
    assertEquals(anonymous.getPrivileges().size(), details.getAuthorities().size());

  }

  @Test
  public void testNoDuplicatePermissions() throws Exception {
    UserDetails details = userDetailsService.loadUserByUsername(Configuration.ANONYMOUS_LOGIN);
    User user = userService.getUserByLogin(Configuration.ANONYMOUS_LOGIN, true);
    assertEquals(user.getPrivileges().size(), details.getAuthorities().size());
  }

}
